@@ -34,11 +34,6 @@ static void fts_xapian_mail_user_created(struct mail_user *user)
         fuser->set.full 	= XAPIAN_DEFAULT_FULL;
 	fuser->set.detach	= 0;
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
-	size_t len = sizeof(fuser->set.pagesize);
-	sysctlbyname("hw.pagesize", &(fuser->set.pagesize), &len, NULL, 0);
-#endif
-
 	const char * env = mail_user_plugin_getenv(user, "fts_xapian");
         if (env == NULL)
         {
