@@ -15,6 +15,7 @@
 #include "mail-storage-hooks.h"
 #include "module-context.h"
 #include "fts-api-private.h"
+#include "master-service.h"
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
 #include <sys/types.h>
@@ -23,16 +24,19 @@
 
 #include <sqlite3.h>
 
-#define XAPIAN_MIN_RAM 250L // MB
+// NGram parameters (default)
 #define XAPIAN_DEFAULT_PARTIAL 3L
 #define XAPIAN_DEFAULT_FULL 20L
 
-#define XAPIAN_FILE_PREFIX "xapian-indexes"
-#define XAPIAN_TERM_SIZELIMIT 245L
-#define XAPIAN_MAXTERMS_PERDOC 50000L
-#define XAPIAN_THREAD_SIZE 3L
-#define XAPIAN_WRITING_CACHE 2000L
+// Ressources limits
+#define XAPIAN_FILE_PREFIX "xapian-indexes" // Locations of indexes
+#define XAPIAN_TERM_SIZELIMIT 245L // Hard limit of Xapian library
+#define XAPIAN_MAXTERMS_PERDOC 50000L // Nb of keywords max per email
+#define XAPIAN_WRITING_CACHE 4000L // Max nb of emails processed in cache 
+#define XAPIAN_MIN_RAM 500L // MB
+#define XAPIAN_MAX_ERRORS 1024L 
 
+// Word processing
 #define XAPIAN_WILDCARD "wldcrd"
 #define XAPIAN_EXPUNGE_HEADER 9
 
@@ -46,11 +50,11 @@ static const char * selectUIDs = "select ID from docs;";
 #define CHAR_KEY "_"
 #define CHAR_SPACE " "
 
-#define CHARS_PB 14
-static const char * chars_pb[] = { "<", ">", ".", "-", "@", "&", "%", "*", "|", "`", "#", "~", "^", "\\" };
+#define CHARS_PB 16
+static const char * chars_pb[] = { "<", ">", ".", "-", "@", "&", "%", "*", "|", "`", "#", "^", "\\", "'", "/", "~" };
 
-#define CHARS_SEP 12
-static const char * chars_sep[] = { "'", "\"", "\r", "\n", "\t", ",", ":", ";", "(", ")", "?", "!" };
+#define CHARS_SEP 13
+static const char * chars_sep[] = { "\"", "\r", "\n", "\t", ",", ":", ";", "(", ")", "?", "!", "¿", "¡" };
 
 
 struct fts_xapian_settings
